"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const react_dom_1 = require("react-dom");
const prop_types_1 = __importDefault(require("prop-types"));
const addEventListener_1 = __importDefault(require("rc-util/lib/Dom/addEventListener"));
function rgbaColor(r, g, b, a) {
    return `rgba(${[r, g, b, a / 100].join(',')})`;
}
class Alpha extends react_1.default.Component {
    constructor(props) {
        super(props);
        this.onMouseDown = e => {
            const x = e.clientX;
            const y = e.clientY;
            this.pointMoveTo({
                x,
                y,
            });
            this.dragListener = addEventListener_1.default(window, 'mousemove', this.onDrag);
            this.dragUpListener = addEventListener_1.default(window, 'mouseup', this.onDragEnd);
        };
        this.onDrag = e => {
            const x = e.clientX;
            const y = e.clientY;
            this.pointMoveTo({
                x,
                y,
            });
        };
        this.onDragEnd = e => {
            const x = e.clientX;
            const y = e.clientY;
            this.pointMoveTo({
                x,
                y,
            });
            this.removeListeners();
        };
        this.getBackground = () => {
            const { red, green, blue } = this.props.color;
            const opacityGradient = `linear-gradient(to right, ${rgbaColor(red, green, blue, 0)} , ${rgbaColor(red, green, blue, 100)})`; // eslint-disable-line max-len
            return opacityGradient;
        };
        this.getPrefixCls = () => {
            return `${this.props.rootPrefixCls}-alpha`;
        };
        this.pointMoveTo = coords => {
            const rect = react_dom_1.findDOMNode(this).getBoundingClientRect();
            const width = rect.width;
            let left = coords.x - rect.left;
            left = Math.max(0, left);
            left = Math.min(left, width);
            const alpha = Math.round(left / width * 100);
            this.props.onChange(alpha);
        };
        this.removeListeners = () => {
            if (this.dragListener) {
                this.dragListener.remove();
                this.dragListener = null;
            }
            if (this.dragUpListener) {
                this.dragUpListener.remove();
                this.dragUpListener = null;
            }
        };
    }
    componentWillUnmount() {
        this.removeListeners();
    }
    render() {
        const prefixCls = this.getPrefixCls();
        return (react_1.default.createElement("div", { className: prefixCls },
            react_1.default.createElement("div", { ref: "bg", className: `${prefixCls}-bg`, style: { background: this.getBackground() } }),
            react_1.default.createElement("span", { style: { left: `${this.props.alpha}%` } }),
            react_1.default.createElement("div", { className: `${prefixCls}-handler`, onMouseDown: this.onMouseDown })));
    }
}
exports.default = Alpha;
Alpha.propTypes = {
    color: prop_types_1.default.object,
    onChange: prop_types_1.default.func,
    rootPrefixCls: prop_types_1.default.string,
    alpha: prop_types_1.default.number,
};
