"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const react_dom_1 = __importDefault(require("react-dom"));
const prop_types_1 = __importDefault(require("prop-types"));
const addEventListener_1 = __importDefault(require("rc-util/lib/Dom/addEventListener"));
const color_1 = __importDefault(require("./helpers/color"));
const WIDTH = 200;
const HEIGHT = 150;
class Board extends react_1.default.Component {
    constructor(props) {
        super(props);
        this.onBoardMouseDown = e => {
            const buttons = e.buttons;
            // only work on left click
            // @see https://developer.mozilla.org/en-US/docs/Web/Events/mousedown
            if (buttons !== 1)
                return;
            this.removeListeners();
            const x = e.clientX;
            const y = e.clientY;
            this.pointMoveTo({ x, y });
            this.dragListener = addEventListener_1.default(window, 'mousemove', this.onBoardDrag);
            this.dragUpListener = addEventListener_1.default(window, 'mouseup', this.onBoardDragEnd);
        };
        this.onBoardTouchStart = e => {
            if (e.touches.length !== 1) {
                return;
            }
            this.removeTouchListeners();
            const x = e.targetTouches[0].clientX;
            const y = e.targetTouches[0].clientY;
            this.pointMoveTo({ x, y });
            this.touchMoveListener = addEventListener_1.default(window, 'touchmove', this.onBoardTouchMove);
            this.touchEndListener = addEventListener_1.default(window, 'touchend', this.onBoardTouchEnd);
        };
        this.onBoardTouchMove = e => {
            if (e.preventDefault) {
                e.preventDefault();
            }
            const x = e.targetTouches[0].clientX;
            const y = e.targetTouches[0].clientY;
            this.pointMoveTo({
                x,
                y,
            });
        };
        this.onBoardTouchEnd = () => {
            this.removeTouchListeners();
        };
        this.onBoardDrag = e => {
            const x = e.clientX;
            const y = e.clientY;
            this.pointMoveTo({
                x,
                y,
            });
        };
        this.onBoardDragEnd = e => {
            const x = e.clientX;
            const y = e.clientY;
            this.pointMoveTo({
                x,
                y,
            });
            this.removeListeners();
        };
        this.getPrefixCls = () => {
            return `${this.props.rootPrefixCls}-board`;
        };
        this.removeTouchListeners = () => {
            if (this.touchMoveListener) {
                this.touchMoveListener.remove();
                this.touchMoveListener = null;
            }
            if (this.touchEndListener) {
                this.touchEndListener.remove();
                this.touchEndListener = null;
            }
        };
        this.removeListeners = () => {
            if (this.dragListener) {
                this.dragListener.remove();
                this.dragListener = null;
            }
            if (this.dragUpListener) {
                this.dragUpListener.remove();
                this.dragUpListener = null;
            }
        };
        /**
         * 移动光标位置到
         * @param  {object} pos X Y 全局坐标点
         */
        this.pointMoveTo = pos => {
            const rect = react_dom_1.default.findDOMNode(this).getBoundingClientRect();
            let left = pos.x - rect.left;
            let top = pos.y - rect.top;
            const rWidth = rect.width || WIDTH;
            const rHeight = rect.height || HEIGHT;
            left = Math.max(0, left);
            left = Math.min(left, rWidth);
            top = Math.max(0, top);
            top = Math.min(top, rHeight);
            const { color } = this.props;
            color.saturation = left / rWidth;
            color.brightness = 1 - top / rHeight;
            this.props.onChange(color);
        };
    }
    componentWillUnmount() {
        this.removeListeners();
        this.removeTouchListeners();
    }
    render() {
        const prefixCls = this.getPrefixCls();
        const color = this.props.color;
        const hueHsv = {
            h: color.hue,
            s: 1,
            v: 1,
        };
        const hueColor = new color_1.default(hueHsv).toHexString();
        const xRel = color.saturation * 100;
        const yRel = (1 - color.brightness) * 100;
        return (react_1.default.createElement("div", { className: prefixCls },
            react_1.default.createElement("div", { className: `${prefixCls}-hsv`, style: { backgroundColor: hueColor } },
                react_1.default.createElement("div", { className: `${prefixCls}-value` }),
                react_1.default.createElement("div", { className: `${prefixCls}-saturation` })),
            react_1.default.createElement("span", { style: { left: `${xRel}%`, top: `${yRel}%` } }),
            react_1.default.createElement("div", { className: `${prefixCls}-handler`, onMouseDown: this.onBoardMouseDown, onTouchStart: this.onBoardTouchStart })));
    }
}
exports.default = Board;
/**
 * hsv
 * h: range(0, 359)
 * s: range(0, 1)
 * v: range(0, 1)
 */
Board.propTypes = {
    color: prop_types_1.default.object,
    onChange: prop_types_1.default.func,
    rootPrefixCls: prop_types_1.default.string,
};
