"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const react_dom_1 = require("react-dom");
const prop_types_1 = __importDefault(require("prop-types"));
const rc_trigger_1 = __importDefault(require("rc-trigger"));
const Panel_1 = __importDefault(require("./Panel"));
const placements_1 = __importDefault(require("./placements"));
const color_1 = __importDefault(require("./helpers/color"));
function refFn(field, component) {
    this[field] = component;
}
function prevent(e) {
    e.preventDefault();
}
class ColorPicker extends react_1.default.Component {
    constructor(props) {
        super(props);
        const alpha = typeof props.alpha === 'undefined'
            ? props.defaultAlpha
            : Math.min(props.alpha, props.defaultAlpha);
        this.state = {
            color: props.color || props.defaultColor,
            alpha,
            open: false,
        };
        const events = [
            'onTriggerClick',
            'onChange',
            'onBlur',
            'getPickerElement',
            'getRootDOMNode',
            'getTriggerDOMNode',
            'onVisibleChange',
            'onPanelMount',
            'setOpen',
            'open',
            'close',
            'focus',
        ];
        events.forEach(e => {
            this[e] = this[e].bind(this);
        });
        this.saveTriggerRef = refFn.bind(this, 'triggerInstance');
    }
    componentWillReceiveProps(nextProps) {
        if (nextProps.color) {
            this.setState({
                color: nextProps.color,
            });
        }
        if (nextProps.alpha !== null && nextProps.alpha !== undefined) {
            this.setState({
                alpha: nextProps.alpha,
            });
        }
    }
    onTriggerClick() {
        this.setState({
            open: !this.state.open,
        });
    }
    onChange(colors) {
        this.setState(Object.assign({}, colors), () => {
            this.props.onChange(this.state);
        });
    }
    onBlur() {
        this.setOpen(false);
    }
    onVisibleChange(open) {
        this.setOpen(open);
    }
    onPanelMount(panelDOMRef) {
        if (this.state.open) {
            setTimeout(() => {
                panelDOMRef.focus();
            }, 1);
        }
    }
    setOpen(open, callback) {
        if (this.state.open !== open) {
            this.setState({
                open,
            }, () => {
                if (typeof callback === 'function')
                    callback();
                const { onOpen, onClose } = this.props;
                if (this.state.open) {
                    onOpen(this.state);
                }
                else {
                    onClose(this.state);
                }
            });
        }
    }
    getRootDOMNode() {
        return react_dom_1.findDOMNode(this);
    }
    getTriggerDOMNode() {
        return react_dom_1.findDOMNode(this.triggerInstance);
    }
    getPickerElement() {
        // const state = this.state;
        return (react_1.default.createElement(Panel_1.default, { onMount: this.onPanelMount, defaultColor: this.state.color, alpha: this.state.alpha, enableAlpha: this.props.enableAlpha, prefixCls: `${this.props.prefixCls}-panel`, onChange: this.onChange, onBlur: this.onBlur, mode: this.props.mode, className: this.props.className }));
    }
    open(callback) {
        this.setOpen(true, callback);
    }
    close(callback) {
        this.setOpen(false, callback);
    }
    focus() {
        if (!this.state.open) {
            react_dom_1.findDOMNode(this).focus();
        }
    }
    render() {
        const props = this.props;
        const state = this.state;
        const classes = [`${props.prefixCls}-wrap`, props.className];
        if (state.open) {
            classes.push(`${props.prefixCls}-open`);
        }
        let children = props.children;
        const [r, g, b] = new color_1.default(this.state.color).RGB;
        const RGBA = [r, g, b];
        RGBA.push(this.state.alpha / 100);
        if (children) {
            children = react_1.default.cloneElement(children, {
                ref: this.saveTriggerRef,
                unselectable: 'unselectable',
                style: {
                    backgroundColor: `rgba(${RGBA.join(',')})`,
                },
                onClick: this.onTriggerClick,
                onMouseDown: prevent,
            });
        }
        const { prefixCls, placement, style, getCalendarContainer, align, animation, disabled, transitionName, } = props;
        return (react_1.default.createElement("div", { className: classes.join(' ') },
            react_1.default.createElement(rc_trigger_1.default, { popup: this.getPickerElement(), popupAlign: align, builtinPlacements: placements_1.default, popupPlacement: placement, action: disabled ? [] : ['click'], destroyPopupOnHide: true, getPopupContainer: getCalendarContainer, popupStyle: style, popupAnimation: animation, popupTransitionName: transitionName, popupVisible: state.open, onPopupVisibleChange: this.onVisibleChange, prefixCls: prefixCls }, children)));
    }
}
exports.default = ColorPicker;
ColorPicker.propTypes = {
    defaultColor: prop_types_1.default.string,
    defaultAlpha: prop_types_1.default.number,
    // can custom
    alpha: prop_types_1.default.number,
    children: prop_types_1.default.node.isRequired,
    className: prop_types_1.default.string,
    color: prop_types_1.default.string,
    enableAlpha: prop_types_1.default.bool,
    mode: prop_types_1.default.oneOf(['RGB', 'HSL', 'HSB']),
    onChange: prop_types_1.default.func,
    onClose: prop_types_1.default.func,
    onOpen: prop_types_1.default.func,
    placement: prop_types_1.default.oneOf(['topLeft', 'topRight', 'bottomLeft', 'bottomRight']),
    prefixCls: prop_types_1.default.string.isRequired,
    style: prop_types_1.default.object,
};
ColorPicker.defaultProps = {
    defaultColor: '#F00',
    defaultAlpha: 100,
    onChange() { },
    onOpen() { },
    onClose() { },
    children: react_1.default.createElement("span", { className: "rc-color-picker-trigger" }),
    className: '',
    enableAlpha: true,
    placement: 'topLeft',
    prefixCls: 'rc-color-picker',
    style: {},
};
