"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const prop_types_1 = __importDefault(require("prop-types"));
const color_1 = __importDefault(require("./helpers/color"));
const Board_1 = __importDefault(require("./Board"));
const Preview_1 = __importDefault(require("./Preview"));
const Ribbon_1 = __importDefault(require("./Ribbon"));
const Alpha_1 = __importDefault(require("./Alpha"));
const Params_1 = __importDefault(require("./Params"));
const classnames_1 = __importDefault(require("classnames"));
function noop() { }
class Panel extends react_1.default.Component {
    constructor(props) {
        super(props);
        this.onSystemColorPickerOpen = e => {
            // only work with broswer which support color input
            if (e.target.type === 'color') {
                this.systemColorPickerOpen = true;
            }
        };
        this.onFocus = () => {
            if (this._blurTimer) {
                clearTimeout(this._blurTimer);
                this._blurTimer = null;
            }
            else {
                this.props.onFocus();
            }
        };
        this.onBlur = () => {
            if (this._blurTimer) {
                clearTimeout(this._blurTimer);
            }
            this._blurTimer = setTimeout(() => {
                // if is system color picker open, then stop run blur
                if (this.systemColorPickerOpen) {
                    this.systemColorPickerOpen = false;
                    return;
                }
                this.props.onBlur();
            }, 100);
        };
        /**
         * 响应 alpha 的变更
         * @param  {Number} alpha Range 0~100
         */
        this.handleAlphaChange = alpha => {
            const { color } = this.state;
            color.alpha = alpha;
            this.setState({
                alpha,
                color,
            });
            this.props.onChange({
                color: color.toHexString(),
                alpha,
            });
        };
        /**
         * color change
         * @param  {Object}  color      tinycolor instance
         * @param  {Boolean} syncParams Sync to <Params />
         */
        this.handleChange = (color) => {
            const { alpha } = this.state;
            color.alpha = alpha;
            this.setState({ color });
            this.props.onChange({
                color: color.toHexString(),
                alpha: color.alpha,
            });
        };
        const alpha = typeof props.alpha === 'undefined'
            ? props.defaultAlpha
            : Math.min(props.alpha, props.defaultAlpha);
        const color = new color_1.default(props.color || props.defaultColor);
        this.state = {
            color,
            alpha,
        };
    }
    componentDidMount() {
        this.props.onMount(this.ref);
    }
    componentWillReceiveProps(nextProps) {
        if (nextProps.color) {
            const color = new color_1.default(nextProps.color);
            this.setState({
                color,
            });
        }
        if (nextProps.alpha !== undefined) {
            this.setState({
                alpha: nextProps.alpha,
            });
        }
    }
    render() {
        const { prefixCls, enableAlpha } = this.props;
        const { color, alpha } = this.state;
        const wrapClasses = classnames_1.default({
            [`${prefixCls}-wrap`]: true,
            [`${prefixCls}-wrap-has-alpha`]: enableAlpha,
        });
        return (react_1.default.createElement("div", { ref: (ref) => this.ref = ref, className: [prefixCls, this.props.className].join(' '), style: this.props.style, onFocus: this.onFocus, onBlur: this.onBlur, tabIndex: "0" },
            react_1.default.createElement("div", { className: `${prefixCls}-inner` },
                react_1.default.createElement(Board_1.default, { rootPrefixCls: prefixCls, color: color, onChange: this.handleChange }),
                react_1.default.createElement("div", { className: wrapClasses },
                    react_1.default.createElement("div", { className: `${prefixCls}-wrap-ribbon` },
                        react_1.default.createElement(Ribbon_1.default, { rootPrefixCls: prefixCls, color: color, onChange: this.handleChange })),
                    enableAlpha &&
                        react_1.default.createElement("div", { className: `${prefixCls}-wrap-alpha` },
                            react_1.default.createElement(Alpha_1.default, { rootPrefixCls: prefixCls, alpha: alpha, color: color, onChange: this.handleAlphaChange })),
                    react_1.default.createElement("div", { className: `${prefixCls}-wrap-preview` },
                        react_1.default.createElement(Preview_1.default, { rootPrefixCls: prefixCls, alpha: alpha, onChange: this.handleChange, onInputClick: this.onSystemColorPickerOpen, color: color }))),
                react_1.default.createElement("div", { className: `${prefixCls}-wrap`, style: { height: 40, marginTop: 6 } },
                    react_1.default.createElement(Params_1.default, { rootPrefixCls: prefixCls, color: color, alpha: alpha, onAlphaChange: this.handleAlphaChange, onChange: this.handleChange, mode: this.props.mode, enableAlpha: this.props.enableAlpha })))));
    }
}
exports.default = Panel;
const validationColor_1 = __importDefault(require("./utils/validationColor"));
Panel.propTypes = {
    alpha: prop_types_1.default.number,
    className: prop_types_1.default.string,
    color: validationColor_1.default,
    defaultAlpha: prop_types_1.default.number,
    defaultColor: validationColor_1.default,
    enableAlpha: prop_types_1.default.bool,
    mode: prop_types_1.default.oneOf(['RGB', 'HSL', 'HSB']),
    onBlur: prop_types_1.default.func,
    onChange: prop_types_1.default.func,
    onFocus: prop_types_1.default.func,
    onMount: prop_types_1.default.func,
    prefixCls: prop_types_1.default.string,
    style: prop_types_1.default.object,
};
Panel.defaultProps = {
    className: '',
    defaultAlpha: 100,
    defaultColor: '#ff0000',
    enableAlpha: true,
    mode: 'RGB',
    onBlur: noop,
    onChange: noop,
    onFocus: noop,
    onMount: noop,
    prefixCls: 'rc-color-picker-panel',
    style: {},
};
