"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const prop_types_1 = __importDefault(require("prop-types"));
const classnames_1 = __importDefault(require("classnames"));
const color_1 = __importDefault(require("./helpers/color"));
const percentage_1 = __importDefault(require("./helpers/percentage"));
const modesMap = ['RGB', 'HSB'];
class Params extends react_1.default.Component {
    constructor(props) {
        super(props);
        this.getChannelInRange = (value, index) => {
            const channelMap = {
                RGB: [[0, 255], [0, 255], [0, 255]],
                HSB: [[0, 359], [0, 100], [0, 100]],
            };
            const mode = this.state.mode;
            const range = channelMap[mode][index];
            let result = parseInt(value, 10);
            if (isNaN(result)) {
                result = 0;
            }
            result = Math.max(range[0], result);
            result = Math.min(result, range[1]);
            return result;
        };
        this.getPrefixCls = () => {
            return `${this.props.rootPrefixCls}-params`;
        };
        this.handleHexBlur = () => {
            const hex = this.state.hex;
            let color = null;
            if (color_1.default.isValidHex(hex)) {
                color = new color_1.default(hex);
            }
            if (color !== null) {
                this.setState({
                    color,
                    hex,
                });
                this.props.onChange(color, false);
            }
        };
        this.handleHexPress = event => {
            const hex = this.state.hex;
            if (event.nativeEvent.which === 13) {
                let color = null;
                if (color_1.default.isValidHex(hex)) {
                    color = new color_1.default(hex);
                }
                if (color !== null) {
                    this.setState({
                        color,
                        hex,
                    });
                    this.props.onChange(color, false);
                }
            }
        };
        this.handleHexChange = event => {
            const hex = event.target.value;
            this.setState({
                hex,
            });
        };
        this.handleModeChange = () => {
            let mode = this.state.mode;
            const modeIndex = (modesMap.indexOf(mode) + 1) % modesMap.length;
            mode = modesMap[modeIndex];
            this.setState({
                mode,
            });
        };
        this.handleAlphaHandler = event => {
            let alpha = parseInt(event.target.value, 10);
            if (isNaN(alpha)) {
                alpha = 0;
            }
            alpha = Math.max(0, alpha);
            alpha = Math.min(alpha, 100);
            this.props.onAlphaChange(alpha);
        };
        this.updateColorByChanel = (channel, value) => {
            const { color } = this.props;
            const { mode } = this.state;
            if (mode === 'HSB') {
                if (channel === 'H') {
                    color.hue = parseInt(value, 10);
                }
                else if (channel === 'S') {
                    color.saturation = parseInt(value, 10) / 100;
                }
                else if (channel === 'B') {
                    color.brightness = parseInt(value, 10) / 100;
                }
            }
            else {
                if (channel === 'R') {
                    color.red = parseInt(value, 10);
                }
                else if (channel === 'G') {
                    color.green = parseInt(value, 10);
                }
                else if (channel === 'B') {
                    color.blue = parseInt(value, 10);
                }
            }
            return color;
        };
        this.handleColorChannelChange = (index, event) => {
            const value = this.getChannelInRange(event.target.value, index);
            const { mode } = this.state;
            const channel = mode[index];
            const color = this.updateColorByChanel(channel, value);
            this.setState({
                hex: color.hex,
                color,
            }, () => {
                this.props.onChange(color, false);
            });
        };
        // 管理 input 的状态
        this.state = {
            mode: props.mode,
            hex: props.color.hex,
            color: props.color, // instanceof tinycolor
        };
    }
    componentWillReceiveProps(nextProps) {
        const { color: nextColor } = nextProps;
        this.setState({
            color: nextColor,
            hex: nextColor.hex,
        });
    }
    render() {
        const prefixCls = this.getPrefixCls();
        const { enableAlpha } = this.props;
        const { mode, color } = this.state;
        const colorChannel = color[mode];
        if (mode === 'HSB') {
            colorChannel[0] = parseInt(colorChannel[0], 10);
            colorChannel[1] = percentage_1.default(colorChannel[1]);
            colorChannel[2] = percentage_1.default(colorChannel[2]);
        }
        const paramsClasses = classnames_1.default({
            [prefixCls]: true,
            [`${prefixCls}-has-alpha`]: enableAlpha,
        });
        return (react_1.default.createElement("div", { className: paramsClasses },
            react_1.default.createElement("div", { className: `${prefixCls}-input` },
                react_1.default.createElement("input", { className: `${prefixCls}-hex`, type: "text", maxLength: "6", onKeyPress: this.handleHexPress, onBlur: this.handleHexBlur, onChange: this.handleHexChange, value: this.state.hex.toLowerCase() }),
                react_1.default.createElement("input", { type: "number", ref: "channel_0", value: colorChannel[0], onChange: this.handleColorChannelChange.bind(null, 0) }),
                react_1.default.createElement("input", { type: "number", ref: "channel_1", value: colorChannel[1], onChange: this.handleColorChannelChange.bind(null, 1) }),
                react_1.default.createElement("input", { type: "number", ref: "channel_2", value: colorChannel[2], onChange: this.handleColorChannelChange.bind(null, 2) }),
                enableAlpha && (react_1.default.createElement("input", { type: "number", value: Math.round(this.props.alpha), onChange: this.handleAlphaHandler }))),
            react_1.default.createElement("div", { className: `${prefixCls}-lable` },
                react_1.default.createElement("label", { className: `${prefixCls}-lable-hex` }, "Hex"),
                react_1.default.createElement("label", { className: `${prefixCls}-lable-number`, onClick: this.handleModeChange }, mode[0]),
                react_1.default.createElement("label", { className: `${prefixCls}-lable-number`, onClick: this.handleModeChange }, mode[1]),
                react_1.default.createElement("label", { className: `${prefixCls}-lable-number`, onClick: this.handleModeChange }, mode[2]),
                enableAlpha && react_1.default.createElement("label", { className: `${prefixCls}-lable-alpha` }, "A"))));
    }
}
exports.default = Params;
Params.propTypes = {
    alpha: prop_types_1.default.number,
    enableAlpha: prop_types_1.default.bool,
    color: prop_types_1.default.object.isRequired,
    mode: prop_types_1.default.oneOf(modesMap),
    onAlphaChange: prop_types_1.default.func,
    onChange: prop_types_1.default.func,
    rootPrefixCls: prop_types_1.default.string,
};
Params.defaultProps = {
    mode: modesMap[0],
    enableAlpha: true,
};
