"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const react_dom_1 = __importDefault(require("react-dom"));
const prop_types_1 = __importDefault(require("prop-types"));
const addEventListener_1 = __importDefault(require("rc-util/lib/Dom/addEventListener"));
class Ribbon extends react_1.default.Component {
    constructor(props) {
        super(props);
        this.onMouseDown = e => {
            const x = e.clientX;
            const y = e.clientY;
            this.pointMoveTo({
                x,
                y,
            });
            this.dragListener = addEventListener_1.default(window, 'mousemove', this.onDrag);
            this.dragUpListener = addEventListener_1.default(window, 'mouseup', this.onDragEnd);
        };
        this.onDrag = e => {
            const x = e.clientX;
            const y = e.clientY;
            this.pointMoveTo({
                x,
                y,
            });
        };
        this.onDragEnd = e => {
            const x = e.clientX;
            const y = e.clientY;
            this.pointMoveTo({
                x,
                y,
            });
            this.removeListeners();
        };
        this.getPrefixCls = () => {
            return `${this.props.rootPrefixCls}-ribbon`;
        };
        this.pointMoveTo = coords => {
            const rect = react_dom_1.default.findDOMNode(this).getBoundingClientRect();
            const width = rect.width;
            let left = coords.x - rect.left;
            left = Math.max(0, left);
            left = Math.min(left, width);
            const huePercent = left / width;
            const hue = huePercent * 360;
            const { color } = this.props;
            color.hue = hue;
            this.props.onChange(color);
        };
        this.removeListeners = () => {
            if (this.dragListener) {
                this.dragListener.remove();
                this.dragListener = null;
            }
            if (this.dragUpListener) {
                this.dragUpListener.remove();
                this.dragUpListener = null;
            }
        };
    }
    componentWillUnmount() {
        this.removeListeners();
    }
    render() {
        const prefixCls = this.getPrefixCls();
        const hue = this.props.color.hue;
        const per = hue / 360 * 100;
        return (react_1.default.createElement("div", { className: prefixCls },
            react_1.default.createElement("span", { ref: "point", style: { left: `${per}%` } }),
            react_1.default.createElement("div", { className: `${prefixCls}-handler`, onMouseDown: this.onMouseDown })));
    }
}
exports.default = Ribbon;
Ribbon.propTypes = {
    rootPrefixCls: prop_types_1.default.string,
    color: prop_types_1.default.object,
    onChange: prop_types_1.default.func,
};
